package com.example.springdemo.controller;

import com.example.springdemo.entities.Users;
import com.example.springdemo.entities.password.UsersPassword;
import com.example.springdemo.service.UsersService;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.ObjectWriter;
import jakarta.annotation.Resource;
import org.jetbrains.annotations.NotNull;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.runner.RunWith;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.security.test.context.support.WithUserDetails;
import org.springframework.test.annotation.Rollback;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;

import java.util.logging.Logger;

// close or open Spring Security configuration
@AutoConfigureMockMvc(addFilters = true)
@WebMvcTest(UsersController.class)
class UsersControllerTests {
    @Resource
    private UsersController usersController;

    @MockBean
    private UsersService usersService;

    @Resource
    private MockMvc mockMvc;

    @BeforeEach
    public void setup() {

    }

    @Test
    public void contextLoads() {
        // Test that the controller loads
        assert usersController != null;
        // Test that the service loads
        assert usersService != null;
    }

    @WithMockUser(roles = {"管理员"})
    @Test
    public void testGetAllUsers() throws Exception {
        MvcResult result = mockMvc.perform(MockMvcRequestBuilders.get("/users/find/all"))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andReturn();
        Logger logger = Logger.getLogger("testGetAllUsers");
        logger.info(result.getResponse().getContentAsString());
    }

    @WithMockUser(roles = {"管理员"})
    @Test
    public void testGetUserByName() throws Exception {
        mockMvc.perform(MockMvcRequestBuilders.get("/users/find/name").param("name", "admin"))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andReturn();
    }

    @WithMockUser(roles = {"用户"})
    @Test
    public void testGetUserById() throws Exception {
        mockMvc.perform(MockMvcRequestBuilders.get("/users/find/id").param("id", "1"))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andReturn();
    }

    @WithMockUser(roles = {"管理员"})
    @Test
    @Rollback(value = true)
    public void testAddUser() throws Exception {
        // Create a user and password
        UsersController.wrapperUserAndPassword usersAndPassword = setWrapperUserAndPassword();

        // Convert to JSON
        ObjectMapper mapper = new ObjectMapper();
        ObjectWriter writer = mapper.writer().withDefaultPrettyPrinter();

        String jsonString = writer.writeValueAsString(usersAndPassword);

        mockMvc.perform(MockMvcRequestBuilders.post("/users/add")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(jsonString))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andReturn();
    }

    @NotNull
    private UsersController.wrapperUserAndPassword setWrapperUserAndPassword() {
        Users user = new Users();
        user.setName("test");
        user.setSex("男");
        user.setPhoneNumber("12345678901");
        user.setAddress("test address");
        UsersPassword userPassword = new UsersPassword();
        userPassword.setPassword("test1234567");
        userPassword.setUsers(user);

        UsersController.wrapperUserAndPassword usersAndPassword = new UsersController.wrapperUserAndPassword();
        usersAndPassword.setUser(user);
        usersAndPassword.setUserPassword(userPassword);
        return usersAndPassword;
    }

    @Test
    @Rollback(value = true)
    public void testDeleteUserById() throws Exception {
        mockMvc.perform(MockMvcRequestBuilders.delete("/users/delete/id").param("id", "2"))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andReturn();
    }

    @Test
    @Rollback(value = true)
    public void testDeleteUserByName() throws Exception {
        mockMvc.perform(MockMvcRequestBuilders.delete("/users/delete/name").param("name", "测试用户"))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andReturn();
    }

    @Test
    @Rollback(value = true)
    public void testUpdateUser() throws Exception {
        // Create a user and password
        Users user = new Users();
        user.setId(2L);
        user.setName("test");
        user.setSex("女");
        user.setPhoneNumber("12345678901");
        user.setAddress("test address");
        UsersPassword userPassword = new UsersPassword();
        userPassword.setPassword("test1234567");
        userPassword.setUsers(user);

        // Convert to JSON
        ObjectMapper mapper = new ObjectMapper();
        ObjectWriter writer = mapper.writer().withDefaultPrettyPrinter();

        String jsonString = writer.writeValueAsString(user);

        mockMvc.perform(MockMvcRequestBuilders.put("/users/update/info")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(jsonString))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andReturn();
    }

    @WithMockUser(username = "测试用户", password = "20211120172", roles = {"用户"})
    @Test
    @Rollback(value = true)
    public void testUpdateUserPassword() throws Exception {
        mockMvc.perform(MockMvcRequestBuilders.put("/users/update/password")
                        .param("password", "test1234567")
                        .param("userId", "2"))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andReturn();
    }
}
